@@ -1,9 +1,11 @@
-import os
+"utilities for getting information about the system, see `SysInfo` class"
+
+from __future__ import annotations
+
 import subprocess
 import sys
 import typing
-
-from pip._internal.operations.freeze import freeze
+from importlib.metadata import distributions
 
 
 def _popen(cmd: list[str], split_out: bool = False) -> dict[str, typing.Any]:
@@ -11,9 +13,11 @@ def _popen(cmd: list[str], split_out: bool = False) -> dict[str, typing.Any]:
         cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
     )
 
-    p_out: str | list[str] | None
-    if p.stdout is not None:
-        p_out = p.stdout.read().decode("utf-8")
+    stdout, stderr = p.communicate()
+
+    p_out: typing.Union[str, list[str], None]
+    if stdout:
+        p_out = stdout.decode("utf-8")
         if split_out:
             assert isinstance(p_out, str)
             p_out = p_out.strip().split("\n")
@@ -22,7 +26,7 @@ def _popen(cmd: list[str], split_out: bool = False) -> dict[str, typing.Any]:
 
     return {
         "stdout": p_out,
-        "stderr": (None if p.stderr is None else p.stderr.read().decode("utf-8")),
+        "stderr": stderr.decode("utf-8") if stderr else None,
         "returncode": p.returncode if p.returncode is None else int(p.returncode),
     }
 
@@ -36,6 +40,7 @@ class SysInfo:
         ver_tup = sys.version_info
         return {
             "version": sys.version,
+            "version_info": ver_tup,
             "major": ver_tup[0],
             "minor": ver_tup[1],
             "micro": ver_tup[2],
@@ -46,7 +51,8 @@ class SysInfo:
     @staticmethod
     def pip() -> dict:
         """installed packages info"""
-        pckgs: list[str] = [x for x in freeze(local_only=True)]
+        # for some reason, python 3.8 thinks `Distribution` has no attribute `name`?
+        pckgs: list[tuple[str, str]] = [(x.name, x.version) for x in distributions()]  # type: ignore[attr-defined]
         return {
             "n_packages": len(pckgs),
             "packages": pckgs,
@@ -100,7 +106,7 @@ class SysInfo:
                                 "device": current_device,
                                 "name": dev_prop.name,
                                 "version": {
-                                    f"major": dev_prop.major,
+                                    "major": dev_prop.major,
                                     "minor": dev_prop.minor,
                                 },
                                 "total_memory": dev_prop.total_memory,
@@ -138,26 +144,32 @@ class SysInfo:
         return {x: getattr(platform, x)() for x in items}
 
     @staticmethod
-    def git_info() -> dict:
+    def git_info(with_log: bool = False) -> dict:
         git_version: dict = _popen(["git", "version"])
         git_status: dict = _popen(["git", "status"])
-        if git_status["stderr"].startswith("fatal: not a git repository"):
+        if not git_status["stderr"] or git_status["stderr"].startswith(
+            "fatal: not a git repository"
+        ):
             return {
                 "git version": git_version["stdout"],
                 "git status": git_status,
             }
         else:
-            return {
+            output: dict = {
+                "git version": git_version["stdout"],
                 "git status": git_status,
                 "git branch": _popen(["git", "branch"], split_out=True),
                 "git remote -v": _popen(["git", "remote", "-v"], split_out=True),
-                "git log": _popen(["git", "log"]),
             }
+            if with_log:
+                output["git log"] = _popen(["git", "log"], split_out=False)
+
+            return output
 
     @classmethod
     def get_all(
         cls,
-        include: tuple[str, ...] | None = None,
+        include: typing.Optional[tuple[str, ...]] = None,
         exclude: tuple[str, ...] = tuple(),
     ) -> dict:
         include_meta: tuple[str, ...]
@@ -179,3 +191,9 @@ class SysInfo:
                 ]
             )
         }
+
+
+if __name__ == "__main__":
+    import pprint
+
+    pprint.pprint(SysInfo.get_all())
